@@ -124,6 +124,7 @@ def get_workspace_info_from_content(fcontent, ftype, md5hash) -> Tuple[Path]:
 def get_workspace_info_from_path(protein_file_path, md5hash, ftype) -> Tuple[Path]:
     workspace_info = get_workspace_info(md5hash, ftype, fresh=True, create=True)
     sh.cp(protein_file_path, workspace_info['protein_file_path'])
+    gen_structure(workspace_info)
     return workspace_info
 
 
@@ -154,7 +155,46 @@ def st_molstar_pockets(protein_file_path, structure_file_path, pockets_file_path
     }
 
 
-def select_pocket_from_local_protein(protein_file_path, *, multi_select=False,  p2rank_home=None, key=None):
+def get_pockets_from_local_protein(protein_file_path, *, p2rank_home=None, preview=True, key=None):
+    p2rank_home = p2rank_home or os.getenv('P2RANK_HOME')
+    assert p2rank_home
+    if isinstance(content, str):
+        content = content.encode('utf-8')
+    md5hash = hashlib.md5(content).hexdigest()
+    
+    ftype = _get_file_type(protein_file_path)
+    workspace_info = get_success_workspace_info(md5hash, ftype)
+    if not workspace_info:    
+        if st.button('Start Discover Pockets'):
+            with st.spinner('wait_for_it'):
+                workspace_info = get_workspace_info_from_content(content, ftype, md5hash)
+                predict_path_p = Path(workspace_info['workdir']) / 'predict'
+                cmd = sh.Command(os.path.join(p2rank_home, 'prank'))
+                args = ['predict', '-f', workspace_info['protein_file_path'], '-o', str(predict_path_p)]
+                cmd(*args, _cwd=p2rank_home, _fg=True)
+                protein_file_name = os.path.basename(workspace_info['protein_file_path'])
+                tmp_pockets_file_path_p = predict_path_p / f'{protein_file_name}_predictions.csv'
+                sh.cp(str(tmp_pockets_file_path_p), workspace_info['pockets_file_path'])
+    if workspace_info:
+        pockets = st_molstar_pockets(workspace_info['protein_file_path'],
+                                     workspace_info['structure_file_path'],
+                                     workspace_info['pockets_file_path'], preview=preview, key=key)
+        return pockets  
+
+
+def select_pocket_from_local_protein(protein_file_path, *, multi_select=False,  preview=True, p2rank_home=None, key=None):
+    pockets = get_pockets_from_local_protein(protein_file_path, p2rank_home=p2rank_home, preview=preview, key=key)
+    if pockets:
+        if multi_select:
+           selected_pockets = st.multiselect('Choose Pocket', pockets.keys(), format_func=lambda x: f"{x} | {pockets[x]}", key=f'{key}-select-box')
+           selected = [pockets[i] for i in selected_pockets]
+        else:
+           selected_pocket = st.selectbox('Choose Pocket', pockets.keys(), format_func=lambda x: f"{x} | {pockets[x]}", key=f'{key}-select-box')
+           selected = pockets[selected_pocket]
+        return selected
+
+
+def get_pockets_from_local_protein(protein_file_path, *, p2rank_home=None, preview=True, key=None):
     p2rank_home = p2rank_home or os.getenv('P2RANK_HOME')
     assert p2rank_home
     with open(protein_file_path, 'rb') as f:
@@ -162,27 +202,22 @@ def select_pocket_from_local_protein(protein_file_path, *, multi_select=False,
         ftype = _get_file_type(protein_file_path)
     workspace_info = get_success_workspace_info(md5hash, ftype)
     if not workspace_info:    
-        if st.button('Process'):
+        if st.button('Start Discover Pockets'):
             with st.spinner('wait_for_it'):
                 workspace_info = get_workspace_info_from_path(protein_file_path, md5hash, ftype)
+                predict_path_p = Path(workspace_info['workdir']) / 'predict'
                 cmd = sh.Command(os.path.join(p2rank_home, 'prank'))
-                args = ['predict', '-f', workspace_info['protein_file_path']]
+                args = ['predict', '-f', workspace_info['protein_file_path'], '-o', str(predict_path_p)]
                 cmd(*args, _cwd=p2rank_home, _fg=True)
                 protein_file_name = os.path.basename(workspace_info['protein_file_path'])
-                bname = os.path.splitext(protein_file_name)[0]
-                tmp_pockets_file_path_p = Path(p2rank_home) / 'test_output' / f'predict_{bname}' / f'{protein_file_name}_predictions.csv'
+                tmp_pockets_file_path_p = predict_path_p / f'{protein_file_name}_predictions.csv'
                 sh.cp(str(tmp_pockets_file_path_p), workspace_info['pockets_file_path'])
     if workspace_info:
         pockets = st_molstar_pockets(workspace_info['protein_file_path'],
                                      workspace_info['structure_file_path'],
-                                     workspace_info['pockets_file_path'], key=key)
-        if multi_select:
-           selected_pockets = st.multiselect('Choose Pocket', pockets.keys(), format_func=lambda x: f"{x} | {pockets[x]}", key=f'{key}-select-box')
-           selected = [pockets[i] for i in selected_pockets]
-        else:
-           selected_pocket = st.selectbox('Choose Pocket', pockets.keys(), format_func=lambda x: f"{x} | {pockets[x]}", key=f'{key}-select-box')
-           selected = pockets[selected_pocket]
-        return workspace_info['protein_file_path'], selected
+                                     workspace_info['pockets_file_path'], preview=preview, key=key)
+        return pockets
+
 
 def select_pocket_from_upload_protein(*, multi_select=False, p2rank_home=None, preview=True, key=None):
     file = st.file_uploader('Protein', type='pdb')
@@ -192,7 +227,7 @@ def select_pocket_from_upload_protein(*, multi_select=False, p2rank_home=None, p
 
 
 def select_pocket_from_protein_content(content, ftype, *, multi_select=False, p2rank_home=None, preview=True, key=None):
-    pockets = get_pockets_from_protein_content(content, ftype, multi_select=multi_select, p2rank_home=p2rank_home, preview=preview, key=key)
+    pockets = get_pockets_from_protein_content(content, ftype, p2rank_home=p2rank_home, preview=preview, key=key)
     if pockets:
         if multi_select:
             selected_pockets = st.multiselect('Choose Pocket', pockets.keys(), format_func=lambda x: f"{x} | {pockets[x]}", key=f'{key}-select-box')
@@ -200,10 +235,10 @@ def select_pocket_from_protein_content(content, ftype, *, multi_select=False, p2
         else:
             selected_pocket = st.selectbox('Choose Pocket', pockets.keys(), format_func=lambda x: f"{x} | {pockets[x]}", key=f'{key}-select-box')
             selected = pockets[selected_pocket]
-    return pockets
+    return selected
 
 
-def get_pockets_from_protein_content(content, ftype, *, multi_select=False, p2rank_home=None, preview=True, key=None):
+def get_pockets_from_protein_content(content, ftype, *, p2rank_home=None, preview=True, key=None):
     p2rank_home = p2rank_home or os.getenv('P2RANK_HOME')
     assert p2rank_home
     if isinstance(content, str):
@@ -215,12 +250,12 @@ def get_pockets_from_protein_content(content, ftype, *, multi_select=False, p2ra
         if st.button('Start Discover Pockets'):
             with st.spinner('wait_for_it'):
                 workspace_info = get_workspace_info_from_content(content, ftype, md5hash)
+                predict_path_p = Path(workspace_info['workdir']) / 'predict'
                 cmd = sh.Command(os.path.join(p2rank_home, 'prank'))
-                args = ['predict', '-f', workspace_info['protein_file_path']]
+                args = ['predict', '-f', workspace_info['protein_file_path'], '-o', str(predict_path_p)]
                 cmd(*args, _cwd=p2rank_home, _fg=True)
                 protein_file_name = os.path.basename(workspace_info['protein_file_path'])
-                bname = os.path.splitext(protein_file_name)[0]
-                tmp_pockets_file_path_p = Path(p2rank_home) / 'test_output' / f'predict_{bname}' / f'{protein_file_name}_predictions.csv'
+                tmp_pockets_file_path_p = predict_path_p / f'{protein_file_name}_predictions.csv'
                 sh.cp(str(tmp_pockets_file_path_p), workspace_info['pockets_file_path'])
     if workspace_info:
         pockets = st_molstar_pockets(workspace_info['protein_file_path'],
@@ -238,6 +273,5 @@ if _DEVELOP_MODE or os.getenv('SHOW_MOLSTAR_DEMO'):
     else:
         selected = select_pocket_from_local_protein("examples/pocket/protein.pdb", p2rank_home='/Users/wfluo/Downloads/p2rank_2.4/')
     if selected:
-        protein_file_path, pocket = selected
-        st.write('Protein Path: ', protein_file_path)
+        pocket = selected
         st.write('Selected Pocket: ', pocket)
\ No newline at end of file
